// Copyright (C) 2024 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

#include <qmldesignercomponents_global.h>

#include <QString>

namespace QmlDesigner {

namespace DialogUtils {

QMLDESIGNERCOMPONENTS_EXPORT void showWarningForInvalidId(const QString &id);

} // namespace DialogUtils
} //QmlDesigner
