// Copyright (C) 2022 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-2.1-or-later OR GPL-3.0-or-later

#pragma once

#include <QCoreApplication>

namespace ADS {

struct Tr
{
    Q_DECLARE_TR_FUNCTIONS(QtC::ADS)
};

} // AdvancedDockingSystem
