// Copyright (C) 2016 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

#include "exception.h"

namespace QmlDesigner {

class CORESHARED_EXPORT ModificationGroupException: public Exception
{
public:
    ModificationGroupException(int line, const QString &function, const QString &file);

    QString type() const;
};

}
