// Copyright (C) 2016 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

#include "projectexplorer_export.h"

#include <utils/icon.h>

namespace ProjectExplorer {
namespace Icons {

PROJECTEXPLORER_EXPORT extern const Utils::Icon BUILD;
PROJECTEXPLORER_EXPORT extern const Utils::Icon BUILD_FLAT;
PROJECTEXPLORER_EXPORT extern const Utils::Icon BUILD_SMALL;
PROJECTEXPLORER_EXPORT extern const Utils::Icon CANCELBUILD_FLAT;
PROJECTEXPLORER_EXPORT extern const Utils::Icon REBUILD;
PROJECTEXPLORER_EXPORT extern const Utils::Icon RUN;
PROJECTEXPLORER_EXPORT extern const Utils::Icon RUN_FLAT;
PROJECTEXPLORER_EXPORT extern const Utils::Icon WINDOW;
PROJECTEXPLORER_EXPORT extern const Utils::Icon DEBUG_START;
PROJECTEXPLORER_EXPORT extern const Utils::Icon DEVICE_READY_INDICATOR;
PROJECTEXPLORER_EXPORT extern const Utils::Icon DEVICE_READY_INDICATOR_OVERLAY;
PROJECTEXPLORER_EXPORT extern const Utils::Icon DEVICE_CONNECTED_INDICATOR;
PROJECTEXPLORER_EXPORT extern const Utils::Icon DEVICE_CONNECTED_INDICATOR_OVERLAY;
PROJECTEXPLORER_EXPORT extern const Utils::Icon DEVICE_DISCONNECTED_INDICATOR;
PROJECTEXPLORER_EXPORT extern const Utils::Icon DEVICE_DISCONNECTED_INDICATOR_OVERLAY;
PROJECTEXPLORER_EXPORT extern const Utils::Icon WIZARD_IMPORT_AS_PROJECT;
PROJECTEXPLORER_EXPORT extern const Utils::Icon CMAKE_LOGO;
PROJECTEXPLORER_EXPORT extern const Utils::Icon CMAKE_LOGO_TOOLBAR;

PROJECTEXPLORER_EXPORT extern const Utils::Icon DEBUG_START_FLAT;
PROJECTEXPLORER_EXPORT extern const Utils::Icon DEBUG_START_SMALL;
PROJECTEXPLORER_EXPORT extern const Utils::Icon DEBUG_START_SMALL_TOOLBAR;
PROJECTEXPLORER_EXPORT extern const Utils::Icon ANALYZER_START_SMALL;
PROJECTEXPLORER_EXPORT extern const Utils::Icon ANALYZER_START_SMALL_TOOLBAR;

PROJECTEXPLORER_EXPORT extern const Utils::Icon BUILDSTEP_MOVEUP;
PROJECTEXPLORER_EXPORT extern const Utils::Icon BUILDSTEP_MOVEDOWN;
PROJECTEXPLORER_EXPORT extern const Utils::Icon BUILDSTEP_DISABLE;
PROJECTEXPLORER_EXPORT extern const Utils::Icon BUILDSTEP_DISABLE_TOOLBAR;
PROJECTEXPLORER_EXPORT extern const Utils::Icon BUILDSTEP_REMOVE;

PROJECTEXPLORER_EXPORT extern const Utils::Icon DESKTOP_DEVICE;

PROJECTEXPLORER_EXPORT extern const Utils::Icon MODE_PROJECT_CLASSIC;
PROJECTEXPLORER_EXPORT extern const Utils::Icon MODE_PROJECT_FLAT;
PROJECTEXPLORER_EXPORT extern const Utils::Icon MODE_PROJECT_FLAT_ACTIVE;

} // namespace Icons
} // namespace ProjectExplorer
