// Copyright (C) 2016 Nicolas Arnaud-Cormos
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

#include <QtGlobal>

namespace Macros {
namespace Constants {

const char M_TOOLS_MACRO[] = "Macros.Tools.Menu";

const char START_MACRO[] = "Macros.StartMacro";
const char END_MACRO[] = "Macros.EndMacro";
const char EXECUTE_LAST_MACRO[] = "Macros.ExecuteLastMacro";
const char SAVE_LAST_MACRO[] = "Macros.SaveLastMacro";
const char PREFIX_MACRO[] = "Macros.";

const char M_OPTIONS_PAGE[] = "Macros";

const char M_STATUS_BUFFER[] = "Macros.Status";

const char M_EXTENSION[] = "mac";

} // namespace Constants
} // namespace Macros
