// Copyright (C) 2018 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

namespace CompilationDatabaseProjectManager::Constants {

const char COMPILATIONDATABASEMIMETYPE[]  = "text/x-compilation-database-project";
const char COMPILATIONDATABASEPROJECT_ID[] = "CompilationDatabase.CompilationDatabaseEditor";
const char COMPILATIONDATABASEPROJECT_FILES_SUFFIX[] = ".files";

} // CompilationDatabaseProjectManager::Constants
