// Copyright (C) 2016 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

namespace QmlDebug {
namespace Constants {

const char STR_WAITING_FOR_CONNECTION[] = "Waiting for connection ";
const char STR_ON_PORT_PATTERN[] = "on port (\\d+)";
const char STR_UNABLE_TO_LISTEN[] = "Unable to listen ";
const char STR_IGNORING_DEBUGGER[] = "Ignoring \"-qmljsdebugger=";
const char STR_CONNECTION_ESTABLISHED[] = "Connection established";
const char STR_CONNECTING_TO_SOCKET[] = "Connecting to socket";

} // namespace Constants
} // namespace QmlDebug
