// Copyright (C) 2019 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

namespace Qdb::Constants {

const char QdbRunConfigurationId[] = "QdbLinuxRunConfiguration:";
const char QdbDeployConfigurationId[] = "Qt4ProjectManager.Qdb.QdbDeployConfiguration";

const char QdbStopApplicationStepId[] = "Qdb.StopApplicationStep";
const char QdbMakeDefaultAppStepId[] = "Qdb.MakeDefaultAppStep";

const char QdbHardwareDevicePrefix[] = "QdbHardwareDevice";
const char AppcontrollerFilepath[] = "/usr/bin/appcontroller";

} // namespace Qdb::Constants
