// Copyright (C) 2020 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

namespace WebAssembly {
namespace Constants {

const char SETTINGS_ID[] = "CC.WebAssembly.Configuration";

const char WEBASSEMBLY_TOOLCHAIN_TYPEID[] = "WebAssembly.ToolChain.Emscripten";
const char WEBASSEMBLY_DEVICE_TYPE[] = "WebAssemblyDeviceType";
const char WEBASSEMBLY_DEVICE_DEVICE_ID[] = "WebAssembly Device";
const char WEBASSEMBLY_QT_VERSION[] = "Qt4ProjectManager.QtVersion.WebAssembly";
const char WEBASSEMBLY_RUNCONFIGURATION_EMRUN[] = "WebAssembly.RunConfiguration.Emrun";

const char WEBASSEMBLY_EMSDK_CONFIG_FILE[] = ".emscripten";

} // namespace WebAssembly
} // namespace Constants
