// Copyright (C) 2016 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

namespace SharedTools {
   namespace Internal {
       enum { SELECTION_HANDLE_SIZE = 6, SELECTION_MARGIN = 10 };
       enum SelectionHandleState { SelectionHandleOff, SelectionHandleInactive, SelectionHandleActive };
   }
}
