// Copyright (C) 2022 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

#include "../3rdparty/tl_expected/include/tl/expected.hpp"

namespace Utils {

using namespace tl;

} // namespace Utils
