// Copyright (C) 2016 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

#include <QtGlobal>

namespace CodePaster {
namespace Constants {

const char CPASTER_SETTINGS_CATEGORY[] = "XZ.CPaster";

} // namespace Constants
} // namespace CodePaster
