// Copyright (C) 2020 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0

#pragma once

namespace IncrediBuild::Constants {

const char INCREDIBUILD_BUILDSTEP_TYPE[] = "IncrediBuild.BuildStep.Type";

const char BUILDCONSOLE_BUILDSTEP_ID[] = "IncrediBuild.BuildStep.BuildConsole";
const char IBCONSOLE_BUILDSTEP_ID[] = "IncrediBuild.BuildStep.IB_Console";

} // IncrediBuild::Constants
